/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2022 . All rights reserved.
*
*/
package net.northking.cctp.dataPool.api.dataRule.service;

import net.northking.cctp.common.db.Pagination;
import net.northking.cctp.common.db.excel.ExcelService;
import net.northking.cctp.common.http.QueryByPage;
import net.northking.cctp.dataPool.dto.dataRule.*;
import net.northking.cctp.dataPool.dto.dataSets.AtuDataSetFeignDto;
import net.northking.cctp.dataPool.dto.dataSets.AtuScriptInputSetQuoteFieldDto;

import java.util.List;
import java.util.Map;

/**
 * 数据规则表Restful逻辑接口
 * @author maven-cctp-plugin
 * @since 1.0
 */
public interface DataRuleApiService extends ExcelService
{
  /**
   * 查询数据规则表列表
   *
   * @param queryByPage  查询条件
   * @return 查询结果列表
   */
  Pagination<DataRuleDetailDto> pagingQuery(QueryByPage<DataRuleQueryDto> queryByPage);

  /**
   * 新增数据规则表
   *
   * @param dto         接口数据传输对象
   * @return 主键
   */
  String add(DataRuleAddDto dto);

  /**
   * 更新数据规则表
   *
   * @param dto         接口数据传输对象
   * @return 修改的记录数量
   */
  Integer updateByPK(DataRuleUpdateDto dto) throws IllegalAccessException;

  /**
   * 获取数据规则表
   * @param uuid 主键
   * @return 接口数据传输对象
   */
  DataRuleDetailDto findByPK(String uuid);

  /**
   * 删除数据规则表
   * @param uuid  主键
   * @return 删除的记录数量
   */
  Integer deleteByPK(String uuid);

  /**
   * 测试规则
   * @param dto
   * @return
   */
    List<String> executeRule(DataRuleExecuteDto dto);

  /**
   * 执行计划根据规则名称取值
   * @param ruleName
   * @param proId
   * @return
   */
  String getRuleDataByName(String ruleName, String proId);

    Map<String, String> queryFuzzyDataNameMap(AtuDataSetFeignDto setFeignDto);

  AtuScriptInputSetQuoteFieldDto queryFuzzyDataQuoteInfo(AtuScriptInputSetQuoteFieldDto atuScriptInputSetQuoteFieldDto);
}
