/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2022 . All rights reserved.
*
*/
package net.northking.cctp.dataPool.dto.dataSets;

import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.Size;
import java.io.Serializable;
import java.util.List;
import java.util.Map;

/**
* Restful接口--新增资源--数据传输对象：数据集
*
* createdate:  2022-10-24 17:01:19  <br>
* @author:  maven-cctp-plugin  <br>
* @since:  1.0 <br>
*/
@JsonSerialize
@ApiModel(description = "数据集")
public class DataSetsAddDto implements Serializable
{
    /**
     * 数据集名称<br>
     */
    @ApiModelProperty("数据集名称")
    @Size(max = 30, message="数据集名称-name: 数据长度不能 > 30" )
    @NotBlank(message = "数据集名称不能为空")
    private String name;
    /**
     * 描述<br>
     */
    @ApiModelProperty("描述")
    @Size(max = 255, message="描述-description: 数据长度不能 > 255" )
    private String description;
    /**
     * 系统id<br>
     */
    @ApiModelProperty("系统id")
    @Size(max = 32, message="系统id-proId: 数据长度不能 > 32" )
    @NotBlank(message = "系统id不能为空")
    private String proId;
    /**
     * 目录id<br>
     */
    @ApiModelProperty("目录id")
    @Size(max = 32, message="目录id-treeId: 数据长度不能 > 32" )
    @NotBlank(message = "目录id不能为空")
    private String treeId;

    @ApiModelProperty("表头列表")
    private List<String> headers;
    @ApiModelProperty("表数据")
    private List<Map<String, Object>> tableData;

    @ApiModelProperty("环境id")
    private String envId;

    public String getEnvId() {
        return envId;
    }

    public void setEnvId(String envId) {
        this.envId = envId;
    }

    public List<String> getHeaders() {
        return headers;
    }

    public void setHeaders(List<String> headers) {
        this.headers = headers;
    }

    public List<Map<String, Object>> getTableData() {
        return tableData;
    }

    public void setTableData(List<Map<String, Object>> tableData) {
        this.tableData = tableData;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public String getProId() {
        return proId;
    }

    public void setProId(String proId) {
        this.proId = proId;
    }

    public String getTreeId() {
        return treeId;
    }

    public void setTreeId(String treeId) {
        this.treeId = treeId;
    }
}