/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2022 . All rights reserved.
*
*/
package net.northking.cctp.scriptcase.api.casedesign.service;

import net.northking.cctp.common.db.excel.ExcelService;
import net.northking.cctp.common.db.Pagination;
import net.northking.cctp.common.http.QueryByPage;
import net.northking.cctp.scriptcase.db.entity.MqSynchronizeTask;
import net.northking.cctp.scriptcase.dto.casedesign.MqSynchronizeTaskAddDto;
import net.northking.cctp.scriptcase.dto.casedesign.MqSynchronizeTaskDetailDto;
import net.northking.cctp.scriptcase.dto.casedesign.MqSynchronizeTaskQueryDto;
import net.northking.cctp.scriptcase.dto.casedesign.MqSynchronizeTaskUpdateDto;

import java.util.List;

/**
 * dw同步表Restful逻辑接口
 * @author maven-cctp-plugin
 * @since 1.0
 */
public interface MqSynchronizeTaskApiService extends ExcelService
{
  /**
   * 查询dw同步表列表
   *
   * @param queryByPage  查询条件
   * @return 查询结果列表
   */
  Pagination<MqSynchronizeTaskDetailDto> pagingQuery(QueryByPage<MqSynchronizeTaskQueryDto> queryByPage);

  /**
   * 新增dw同步表
   *
   * @param dto         接口数据传输对象
   * @return 主键
   */
  String add(MqSynchronizeTaskAddDto dto);

  /**
   * 更新dw同步表
   *
   * @param dto         接口数据传输对象
   * @return 修改的记录数量
   */
  Integer updateByPK(MqSynchronizeTaskUpdateDto dto) throws IllegalAccessException;

  /**
   * 获取dw同步表
   * @param uuid 主键
   * @return 接口数据传输对象
   */
  MqSynchronizeTaskDetailDto findByPK(String uuid);

  /**
   * 删除dw同步表
   * @param uuid  主键
   * @return 删除的记录数量
   */
  Integer deleteByPK(String uuid);

    /**
     * 批量新增
     *
     * @param batch 集合
     * @return 成功的数量
     */
    List<String> addBatch(List<MqSynchronizeTaskAddDto> batch);

    /**
     * 批量更新
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer updateBatch(List<MqSynchronizeTaskUpdateDto> batch) throws IllegalAccessException;

    /**
     * 批量删除
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer deleteBatch(List<Object> batch);

    List<MqSynchronizeTask> findByTypes(List<Integer> types);
}
