/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2021 . All rights reserved.
*
*/
package net.northking.cctp.testbug.api;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import net.northking.cctp.common.db.Pagination;
import net.northking.cctp.common.http.QueryByPage;
import net.northking.cctp.common.http.ResultWrapper;

import net.northking.cctp.testbug.api.service.BugCorrelationLinkApiService;
import net.northking.cctp.testbug.db.entity.BugCorrelationLink;
import net.northking.cctp.testbug.dto.BugCorrelationLinkDetailDto;
import net.northking.cctp.testbug.dto.BugCorrelationLinkQueryDto;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

@Api(tags = "缺陷关联表")
@RequestMapping("/v1")
@RestController
public class BugCorrelationLinkQueryCtrl
{
  /**
   * 日志
   */
  private final static Logger logger = LoggerFactory.getLogger(BugCorrelationLinkQueryCtrl.class);

  @Autowired
  private BugCorrelationLinkApiService apiService;

  /**
   * 分页查询 缺陷关联表 列表
   *
   * @param queryByPage  查询条件
   * @return 数据列表
   */
  @ApiOperation(value = "分页查询列表：缺陷关联表")
  @PostMapping(value = "/paging/query/BugCorrelationLinks",
      produces = MediaType.APPLICATION_JSON_UTF8_VALUE,
      consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
  public ResultWrapper<Pagination<BugCorrelationLinkDetailDto>> pagingQuery(
      @RequestBody @Validated QueryByPage<BugCorrelationLinkQueryDto> queryByPage)
  {
    ResultWrapper<Pagination<BugCorrelationLinkDetailDto>> wrapper = new ResultWrapper<>();
    Pagination<BugCorrelationLinkDetailDto> page = apiService.pagingQuery(queryByPage);

    return wrapper.success(page);
  }

  @ApiOperation(value = "导出Excel：缺陷关联表")
  @PostMapping(value = "/excel/export/BugCorrelationLinks",
      produces = MediaType.APPLICATION_OCTET_STREAM_VALUE,
      consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
  public void exportExcel(
      @RequestBody @Validated QueryByPage<BugCorrelationLinkQueryDto> queryByPage,
      HttpServletResponse response)
  {
    // 定义导出的Excel在客户端浏览器上提示下载的文件名，不写后缀
    String fileName = "CCTP-缺陷关联表";
    apiService.exportExcel(response, queryByPage, fileName);
  }

  @ApiOperation(value = "下载Excel模板：缺陷关联表")
  @PostMapping(value = "/excel/template/BugCorrelationLinks",
      produces = MediaType.APPLICATION_OCTET_STREAM_VALUE,
      consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
  public void downloadTemplate(HttpServletResponse response)
  {
    String fileName = "CCTP模板-缺陷关联表";
    apiService.downloadTemplate(response, fileName);
  }

  @ApiOperation(value = "导入Excel：缺陷关联表")
  @PostMapping(value = "/excel/import/BugCorrelationLinks",
      produces = MediaType.APPLICATION_JSON_UTF8_VALUE,
      consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
  public ResultWrapper<Integer> importExcel(
      @RequestParam("file") MultipartFile file)
  {
    ResultWrapper<Integer> wrapper = new ResultWrapper<>();
    int count = apiService.importExcel(file);

    return wrapper.success(count);
  }

  /**
   * 列表查询 缺陷关联表
   *
   * @param linkId 关联id
   * @return 数据列表
   */
  @ApiOperation(value = "列表查询：缺陷关联表")
  @PostMapping(value = "/paging/query/BugCorrelationLinkList",
          produces = MediaType.APPLICATION_JSON_UTF8_VALUE,
          consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
  public ResultWrapper<List<BugCorrelationLink>> listQuery(
          @RequestBody @Validated String linkId)
  {
    ResultWrapper<List<BugCorrelationLink>> wrapper = new ResultWrapper<>();
    List<BugCorrelationLink> BugCorrelationLink=apiService.listQuery(linkId);

    return wrapper.success(BugCorrelationLink);
  }
}


