/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2021 . All rights reserved.
*
*/
package net.northking.cctp.dw.api;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import net.northking.cctp.common.db.Pagination;
import net.northking.cctp.common.http.QueryByPage;
import net.northking.cctp.common.http.ResultWrapper;

import net.northking.cctp.common.security.authentication.NKSecurityContext;
import net.northking.cctp.dw.api.service.BugOverallTendencyChartApiService;
import net.northking.cctp.dw.dto.BugOverallTendencyChartDetailDto;
import net.northking.cctp.dw.dto.BugOverallTendencyChartQueryDto;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

@Api(tags = "跨系统缺陷趋势图")
@RequestMapping("/v1")
@RestController
public class BugOverallTendencyChartQueryCtrl
{
  /**
   * 日志
   */
  private final static Logger logger = LoggerFactory.getLogger(BugOverallTendencyChartQueryCtrl.class);

  @Autowired
  private BugOverallTendencyChartApiService apiService;

  /**
   * 查询用户列表
   *
   * @param queryByPage  查询条件
   * @return 数据列表
   */
  @ApiOperation(value = "分页查询列表：跨系统缺陷趋势图")
  @PostMapping(value = "/paging/query/BugOverallTendencyCharts",
      produces = MediaType.APPLICATION_JSON_UTF8_VALUE,
      consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
  public ResultWrapper<Pagination<BugOverallTendencyChartDetailDto>> pagingQuery(
      @RequestBody @Validated QueryByPage<BugOverallTendencyChartQueryDto> queryByPage)
  {
    ResultWrapper<Pagination<BugOverallTendencyChartDetailDto>> wrapper = new ResultWrapper<>();
    Pagination<BugOverallTendencyChartDetailDto> page = apiService.pagingQuery(queryByPage);

    return wrapper.success(page);
  }

  @ApiOperation(value = "导出Excel：跨系统缺陷趋势图")
  @PostMapping(value = "/excel/export/BugOverallTendencyCharts",
      produces = MediaType.APPLICATION_OCTET_STREAM_VALUE,
      consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
  public void exportExcel(
      @RequestBody @Validated QueryByPage<BugOverallTendencyChartQueryDto> queryByPage,
      HttpServletResponse response)
  {
    // 定义导出的Excel在客户端浏览器上提示下载的文件名，不写后缀
    String fileName = "CCTP-跨系统缺陷趋势图";
    apiService.exportExcel(response, queryByPage, fileName);
  }

  @ApiOperation(value = "下载Excel模板：跨系统缺陷趋势图")
  @PostMapping(value = "/excel/template/BugOverallTendencyCharts",
      produces = MediaType.APPLICATION_OCTET_STREAM_VALUE,
      consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
  public void downloadTemplate(HttpServletResponse response)
  {
    String fileName = "CCTP模板-跨系统缺陷趋势图";
    apiService.downloadTemplate(response, fileName);
  }

  @ApiOperation(value = "导入Excel：跨系统缺陷趋势图")
  @PostMapping(value = "/excel/import/BugOverallTendencyCharts",
      produces = MediaType.APPLICATION_JSON_UTF8_VALUE,
      consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
  public ResultWrapper<Integer> importExcel(
      @RequestParam("file") MultipartFile file)
  {
    ResultWrapper<Integer> wrapper = new ResultWrapper<>();
    int count = apiService.importExcel(file);

    return wrapper.success(count);
  }
  //备用接口
  @ApiOperation("列表查询：查询缺陷趋势图信息")
  @PostMapping(value = "/listQuery",
          produces = MediaType.APPLICATION_JSON_UTF8_VALUE,
          consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
  public ResultWrapper<List<BugOverallTendencyChartDetailDto>> listQuery(@RequestBody BugOverallTendencyChartQueryDto bugOverallTendencyChartQueryDto) {
    ResultWrapper<List<BugOverallTendencyChartDetailDto>> wrapper = new ResultWrapper<>();
    bugOverallTendencyChartQueryDto.setTenantId(NKSecurityContext.getTenantId());
    List<BugOverallTendencyChartDetailDto> bugOverallTendencyChartDetailDtos =apiService.listQuery(bugOverallTendencyChartQueryDto);

    bugOverallTendencyChartDetailDtos.forEach(bugOverallTendencyChartDetailDto->{
      bugOverallTendencyChartDetailDto.setViewTime(bugOverallTendencyChartDetailDto.getCreatedTime());
      bugOverallTendencyChartDetailDto.setCreatedTime(null);
    });
    return wrapper.success(bugOverallTendencyChartDetailDtos);
  }
}


