/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2022 . All rights reserved.
*
*/
package net.northking.cctp.dw.api.atu.service;

import net.northking.cctp.common.db.excel.ExcelService;
import net.northking.cctp.common.db.Pagination;
import net.northking.cctp.common.http.QueryByPage;
import net.northking.cctp.dw.db.entity.DwAtuScriptInfo;
import net.northking.cctp.dw.dto.atu.*;

import java.util.List;

/**
 * 脚本信息同步表Restful逻辑接口
 * @author maven-cctp-plugin
 * @since 1.0
 */
public interface DwAtuScriptInfoApiService extends ExcelService
{
  /**
   * 查询脚本信息同步表列表
   *
   * @param queryByPage  查询条件
   * @return 查询结果列表
   */
  Pagination<DwAtuScriptInfoDetailDto> pagingQuery(QueryByPage<DwAtuScriptInfoQueryDto> queryByPage);

  /**
   * 新增脚本信息同步表
   *
   * @param dto         接口数据传输对象
   * @return 主键
   */
  String add(DwAtuScriptInfo dto);

  /**
   * 更新脚本信息同步表
   *
   * @param dto         接口数据传输对象
   * @return 修改的记录数量
   */
  Integer updateByPK(DwAtuScriptInfoUpdateDto dto) throws IllegalAccessException;

  /**
   * 获取脚本信息同步表
   * @param uuid 主键
   * @return 接口数据传输对象
   */
  DwAtuScriptInfoDetailDto findByPK(String uuid);

  /**
   * 删除脚本信息同步表
   * @param uuid  主键
   * @return 删除的记录数量
   */
  Integer deleteByPK(String uuid);

    /**
     * 批量新增
     *
     * @param batch 集合
     * @return 成功的数量
     */
    List<String> addBatch(List<DwAtuScriptInfo> batch);

    /**
     * 批量更新
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer updateBatch(List<DwAtuScriptInfoUpdateDto> batch) throws IllegalAccessException;

    /**
     * 批量删除
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer deleteBatch(List<Object> batch);

  DwAtuDetailTmpDto countByGroup(DwAtuScriptInfoQueryDto query);
}
