/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2021 . All rights reserved.
*
*/
package net.northking.cctp.dw.api.service;

import net.northking.cctp.common.db.excel.ExcelService;
import net.northking.cctp.common.db.Pagination;
import net.northking.cctp.common.http.QueryByPage;
import net.northking.cctp.dw.dto.*;

import java.util.List;

/**
 * 需求分析统计图Restful逻辑接口
 * @author maven-cctp-plugin
 * @since 1.0
 */
public interface MtTestPointStatisticsApiService extends ExcelService
{
  /**
   * 查询需求分析统计图列表
   *
   * @param queryByPage  查询条件
   * @return 查询结果列表
   */
  Pagination<MtTestPointStatisticsDetailDto> pagingQuery(QueryByPage<MtTestPointStatisticsQueryDto> queryByPage);

  /**
   * 新增需求分析统计图
   *
   * @param dto         接口数据传输对象
   * @return 主键
   */
  String add(MtTestPointStatisticsAddDto dto);

  /**
   * 更新需求分析统计图
   *
   * @param dto         接口数据传输对象
   * @return 修改的记录数量
   */
  Integer updateByPK(MtTestPointStatisticsUpdateDto dto) throws IllegalAccessException;

  /**
   * 获取需求分析统计图
   * @param uuid 主键
   * @return 接口数据传输对象
   */
  MtTestPointStatisticsDetailDto findByPK(String uuid);

  /**
   * 删除需求分析统计图
   * @param uuid  主键
   * @return 删除的记录数量
   */
  Integer deleteByPK(String uuid);

    /**
     * 批量新增
     *
     * @param batch 集合
     * @return 成功的数量
     */
    List<String> addBatch(List<MtTestPointStatisticsAddDto> batch);

    /**
     * 批量更新
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer updateBatch(List<MtTestPointStatisticsUpdateDto> batch) throws IllegalAccessException;

    /**
     * 批量删除
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer deleteBatch(List<Object> batch);

    List<MtTestPointStatisticsDetailDto> listQuery(ChartDataGeneQueryDto chartDataGeneQueryDto);
}
